//This code is used to zip, encode, encrypt, then decrypt, decode and unzip all the files in a folder.
//The following must be added under the public tag:
//    public
//    procedure ListFiles();
//    procedure Zip();
//    procedure UnZip();
//    procedure UnZip1Click(Sender: TObject);
//    procedure CEncryptBtnClick(Pp : string; EnInput : string; EnOutput : string);
//    procedure CDecryptBtnClick(Pp : string; DeInput : string; DeOutput : string);
//    procedure Encrypt1Click(Sender: TObject);
//    procedure Decrypt1Click(Sender: TObject);
//    procedure Scrub1Click(Sender: TObject);
//The following must be placed in the global var area:
//var
//   Form1: TForm1;  {data}
//     InvalidPassPhrase : integer = 0;
//     SavePassword : string = '';


//Get list of files in .UserKeys
procedure TForm1.ListFiles();
var SrchRec: TSearchRec;
begin
     Form1.ListBox1.Clear;

     if FindFirst(Application.Location + '.UserKeys\*', faDirectory, SrchRec) = 0 then begin
        try
        repeat
        if not (SrchRec.Name = '.') and not (SrchRec.name = '..') and ((SrchRec.attr and faDirectory) <> faDirectory) then
           Form1.ListBox1.Items.Add(Application.Location + '.UserKeys\' + SrchRec.Name);  // list files

        until
           FindNext(SrchRec) <> 0;

        finally
           FindClose(SrchRec);
        end;
     end;
end;

//Zip file
procedure TForm1.Zip();
var OurZipper: TZipper; x: Integer;
begin
     ListFiles();  //get list of files in .UserKeys
     OurZipper := TZipper.Create;

     try
       OurZipper.FileName := Application.Location + '.UserKeys\archive.zip';

       for x := 0 to ListBox1.Items.Count - 1 do
           if trim(ListBox1.Items.Strings[x]) <> '' then
              OurZipper.Entries.AddFileEntry(ListBox1.Items.Strings[x]);

       OurZipper.ZipAllFiles;
     finally
       OurZipper.Free;
     end;
end;

//UnZip file (files are replaced if already in folder)
procedure TForm1.UnZip();
var UnZipper: TUnZipper;
begin
     UnZipper := TUnZipper.Create;
     try
       UnZipper.FileName := Application.Location + '.UserKeys\archive.zip';
       UnZipper.Examine;
       UnZipper.UnZipAllFiles;
     finally
       UnZipper.Free;
     end;
end;

//UnZip
procedure TForm1.UnZip1Click(Sender: TObject);
begin
     UnZip();
     ScrubFile(Application.Location + '.UserKeys\archive.zip', 3);
     DeleteFile(Application.Location + '.UserKeys\archive.zip');
end;

//Encrypt Passphrases
procedure TForm1.CEncryptBtnClick(Pp : string; EnInput : string; EnOutput : string);
var Source, Dest: file;
    Buffer: array[0..16383] of byte;
    Hash: TDCP_sha256;
    HashDigest: array[0..63] of byte;
    Encrypt: TDCP_twofish;
    Read: integer;
begin
     AssignFile(Source, EnInput);
     Reset(Source, 1);
     AssignFile(Dest, EnOutput);
     Rewrite(Dest, 1);

     ZeroMemory(@Buffer, sizeof(Buffer));
     ZeroMemory(@HashDigest, sizeof(HashDigest));

     FillChar(HashDigest,Sizeof(HashDigest), $FF);   // fill the digest with $FF as the actual digest may not fill the entire digest
     Hash:= TDCP_sha256.Create(Self);
     Hash.Init;
     Hash.UpdateStr(Pp);                     // put passphrase here and hash it to get the key
     Hash.Final(HashDigest);
     Hash.Free;

     Encrypt := (DCP_twofish1);                             // get the component from the combo box
     Encrypt.Init(HashDigest, Sizeof(HashDigest) * 4, nil);     // initialize the cipher with the digest, IV= nil to generate one automatically (note: size in bits ie. sizeof(x)*8)
     Encrypt.EncryptCBC(HashDigest, HashDigest, Sizeof(HashDigest));  // encrypt the hash and write it to the file to use as passphrase
     Encrypt.Reset;

     BlockWrite(Dest, HashDigest, Sizeof(HashDigest));
     repeat
       BlockRead(Source, Buffer, Sizeof(Buffer), Read);
       Encrypt.EncryptCBC(Buffer, Buffer, Read);                      // read from the source encrypt and write out the result
       BlockWrite(Dest, Buffer, Read);
     until Read <> Sizeof(Buffer);
     Encrypt.Burn;                                                  // burn the key data (note: don't free it as we placed it on the form at design time)
     CloseFile(Source);
     CloseFile(Dest);
end;

//   Point1

//Decrypt Passphrases
procedure TForm1.CDecryptBtnClick(Pp : string; DeInput : string; DeOutput : string);
  function CompareMem(I1, I2: PByte; Size: integer): boolean;
  begin
       Result := true;

       repeat
         if I1^<> I2^ then begin
            Result:= false;
            Exit;
            end;

         Inc(I1);
         Inc(I2);
         Dec(Size);
       until Size= 0;
  end;

var Source, Dest: file;
    Buffer: array[0..16383] of byte;
    Hash: TDCP_sha256;
    HashDigest, HashRead: array[0..63] of byte;
    Decrypt: TDCP_twofish;
    Read: integer;
begin
     AssignFile(Source, DeInput);
     Reset(Source, 1);
     AssignFile(Dest, DeOutput);
     Rewrite(Dest, 1);

     ZeroMemory(@Buffer, sizeof(Buffer));
     ZeroMemory(@HashDigest, sizeof(HashDigest));
     InvalidPassPhrase := 0;

     FillChar(HashDigest, Sizeof(HashDigest), $FF);   // fill the digest with $FF as the actual digest may not fill the entire digest
     Hash:= TDCP_sha256.Create(Self);
     Hash.Init;                                     // put passphrase here and hash it to get the key
     Hash.UpdateStr(Pp);
     Hash.Final(HashDigest);
     Hash.Free;
     Decrypt:= DCP_twofish1; // get the component from the combo box
     Decrypt.Init(HashDigest, Sizeof(HashDigest) * 4, nil);                   // initialize the cipher with the digest, IV= nil to generate one automatically (note: size in bits ie. sizeof(x)*8)
     Decrypt.EncryptCBC(HashDigest, HashDigest, Sizeof(HashDigest));        // encrypt the hash to use as confirmation
     Decrypt.Reset;
     BlockRead(Source,HashRead, Sizeof(HashRead), Read);                         // read the other hash from the file and compare it

     if not CompareMem(@HashRead, @HashDigest, Sizeof(HashRead)) then begin
        CloseFile(Source);
        CloseFile(Dest);
        Decrypt.Burn;
        InvalidPassPhrase := 1;
////        MessageDlg('Incorrect passphrase', mtInformation, [mbOK], 0);
        Exit;
        end;
     repeat
       BlockRead(Source, Buffer, Sizeof(Buffer), Read);
       Decrypt.DecryptCBC(Buffer, Buffer, Read);                            // read from the source decrypt and write out the result
       BlockWrite(Dest, Buffer, Read);
     until Read<> Sizeof(Buffer);

     Decrypt.Burn;                                                        // burn the key data (note: don't free it as we placed it on the form at design time)

     CloseFile(Source);
     CloseFile(Dest);
end;

//Encrypt zipped archive
procedure TForm1.Encrypt1Click(Sender: TObject);
begin
     Form1.CEncryptBtnClick(SavePassword, // TwoFishPwd1.Text,                        //encrypt message
           {Input}  ExtractFilePath(Application.ExeName) + '.UserKeys\archive.zip',
           {Output} ExtractFilePath(Application.ExeName) + '.UserKeys\archive.asc');
end;

//Decrypt zipped archive
procedure TForm1.Decrypt1Click(Sender: TObject);
begin
     Form1.CDecryptBtnClick(SavePassword,                        //encrypt message
           {Input}  ExtractFilePath(Application.ExeName) + '.UserKeys\archive.asc',
           {Output} ExtractFilePath(Application.ExeName) + '.UserKeys\archive.zip');

     if InvalidPassPhrase = 0 then begin
        ScrubFile(Application.Location + '.UserKeys\archive.asc', 3);
        DeleteFile(Application.Location + '.UserKeys\archive.asc');
        end
        else
        DeleteFile(Application.Location + '.UserKeys\archive.zip');
        end;

//Scrub files
procedure TForm1.Scrub1Click(Sender: TObject);
var x : integer;
begin
     for x := 0 to ListBox1.Items.Count - 1 do
         if ListBox1.Items.Strings[x] <> '' then begin
         ScrubFile(ListBox1.Items.Strings[x], 3);
         DeleteFile(ListBox1.Items.Strings[x]);
         end;

     ScrubFile(Application.Location + '.UserKeys\archive.zip', 3);
     DeleteFile(Application.Location + '.UserKeys\archive.zip');
end;
